<?php
/*
Plugin Name: trigami nofollow
Plugin URI: http://www.awokenmind.de/2007-12-01/trigami-nofollow
Description: Searches for links to trigami.com and adds a rel="nofollow" tag. If a trigami-link was found in a post EVERY following link in this post will be equipped with a rel="nofollow" tag.
Author: Andreas Brett (based on 'Wikipedia nofollow' by Key Yasumoto-Nicolson and 'Identify External Links' by Mark Jaquith)
Version: 1.0.2
Author URI: http://www.awokenmind.de
*/

/* Based on:
Plugin Name: Wikipedia nofollow
Plugin URI: http://whatjapanthinks.com/wikipedia-nofollow/
Description: Searches for links to Wikipedia.org, and adds a rel="nofollow" tag if necessary
Author: Ken Yasumoto-Nicolson (based on Identify External Links by Mark Jaquith)
Version: 1.0
Author URI: http://whatjapanthinks.com
*/

/* Based on:
Plugin Name: Identify External Links
Plugin URI: http://txfx.net/code/wordpress/identify-external-links/
Description: Searches the text for links outside of the domain of the blog.	 To these, it adds <strong>class="extlink"</strong> (and optionally, <strong>target="_blank"</strong>).
Author: Mark Jaquith
Version: 1.3
Author URI: http://txfx.net/
*/

/*	Copyright 2005	Mark Jaquith (email: mark.gpl@txfx.net)
   Additional changes Copyright 2007 Ken Yasumoto-Nicolson (email: seron@whatjapanthinks.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// if you just want to parse posts in a specific wordpress-category, set this parameter. otherwise leave it blank and every post is parsed
$trigamiCategory = "";

function wp_get_domain_name_from_uri($uri) {
	preg_match("/^(http:\/\/)?([^\/]+)/i", $uri, $matches);
	$host = $matches[2];
	preg_match("/[^\.\/]+\.[^\.\/]+$/", $host, $matches);
	return $matches[0];	   
}

function wp_has_no_rel_nofollow($text) {
	if ( preg_match("/rel=[\"\'].*?nofollow.*?[\"\']/i", $text ) )
		return false;
	else
		return true;
}

function parse_trigami_nofollow($matches) {
	global $isTrigamiPost;
	
	// if we haven't already detected a trigamiPost and this link points to trigami.com => trigamiPost DETECTED
	if (!$isTrigamiPost && wp_get_domain_name_from_uri($matches[3]) == "trigami.com") {
		$isTrigamiPost = true;
	}
	
	// if we have detected a trigamiPost and the processed link has no rel="nofollow" attribute => embed the nofollow-attribute into the link
	if($isTrigamiPost && wp_has_no_rel_nofollow($matches[1]) && wp_has_no_rel_nofollow($matches[4])) {
		return '<a rel="nofollow" href="' . $matches[2] . '//' . $matches[3] . '"' . $matches[1] . $matches[4] . '>' . $matches[5] . '</a>';
	}
	
	// just return the link as we got it before
	else {
		return '<a href="' . $matches[2] . '//' . $matches[3] . '"' . $matches[1] . $matches[4] . '>' . $matches[5] . '</a>';
	}
}
	
function wp_trigami_nofollow($text) {
	global $trigamiCategory;
	
	// we assume to parse every post
	$trigami_nofollow = true;
	
	// but if the user wants it, we have to check this post for its categories before parsing it
	if($trigamiCategory != "") {
		$trigami_nofollow = false;
		foreach((get_the_category()) as $cat) { 
			if(strpos($cat->cat_name, $trigamiCategory) !== false) {
				$trigami_nofollow = true;
				break;
			}
		}
	}
	
	// finally we may parse this post
	if($trigami_nofollow) {
		$pattern = '/<a (.*?)href=[\"\'](.*?)\/\/(.*?)[\"\'](.*?)>(.*?)<\/a>/i';
		$text = preg_replace_callback($pattern, 'parse_trigami_nofollow', $text);
	}
	return $text;
}

// filters have high priority to make sure that any markup plugins like Textile or Markdown have already created the HTML links
$isTrigamiPost = false;
add_filter('the_content', 'wp_trigami_nofollow', 999);
add_filter('the_excerpt', 'wp_trigami_nofollow', 999);
?>